.include "nes.inc"

;.importzp temp
.export the42melody

.segment "CODE"

;;
; The Forty-Two Melody for NES
; https://countercomplex.blogspot.com/2011/10/some-deep-analysis-of-one-line-music.html
; APU_DMC_RAW = t*(42&t>>10)
; At 223 cycles, the sample rate is about 8026hz on NTSC.
; - Position independent executable!
; - Uses 2 bytes of scratch space.
; - Time code is in the X and Y registers.
; - only crashes if jumped into byte ofest 35 (high byte of APU_DMC_RAW)
.proc the42melody
; $88 and $aa are non destructive opcodes, but easy to type with ACE terminal.
temp_0 = $88
temp_1 = $aa
  next_sample:
; X is waveform phase
    stx temp_0
; Upper 5 bits of Y is the note sequence.
    tya
    lsr
    lsr
; Select which of the 8 frequencies to play. 0 being rest.
    and #42
; The loop terminator for the multiply operation.
    ora #%10000000
    sta temp_1

; caculate wave sample by multiplying temp+0 and temp+1 into A
    lda #0
; start in the middle of the loop for branch optimization.
    beq start_loop
    multiply_loop:
; A NOP to correct the total sample rate.
; We choose CLD just in case the decimel flag Interferes.
      cld
; add temp+0 to A when a bit in temp+1 is set.
      bcs cycle_correction_1
        nop
        bcc *+2
      cycle_correction_1:
      bcc skip_add
        clc
        adc temp_0
      skip_add:
; multiply operand by 2 each time a LSB of temp+1 is read.
      asl temp_0
    start_loop:
      lsr temp_1
; the terminateing bit is not processed.
    bne multiply_loop

; output the 7-bit sample.
    lsr
    sta APU_DMC_RAW  ; can crash if we jump into the $40 byte of this instruction

; increment the 16-bit timer, and do the next sample.
    inx
    beq cycle_correction_2
      nop
    cycle_correction_2:
    bne skip_inc_hi
      iny
    skip_inc_hi:
;    clc
;  bcc next_sample
; Replace the last two lines with this to terminate after 1 loop.
    nop
  bne next_sample
rts
.endproc

