.include "nes.inc"

.importzp temp
.importzp tv_type
.importzp ppustrings_ready, ppustrings_count
;.importzp ntd_stream_ptr
.importzp ppu_var_scroll_x, ppu_var_scroll_y, ppu_var_ctrl, ppu_var_mask
.importzp pad_1_keys, pad_2_keys, pad_1_axlr, pad_2_axlr
.import ppu_oam, ppu_buffer

.import ppu_vsync, ppu_set_addr, pads_read, ppu_vsync_forced_blank
.import ppustrings_nmi_kernel
.import ppustrings_init, ppustrings_init_with_callback
.import ppustrings_new_string, ppustrings_finish_string, ppustrings_write_run
.import bagel_bulk_upload
.import ntd_decode
.import stack_saveState, stack_restoreState

.export controller_music

pad_1_inv_prev = pad_1_axlr
pad_2_inv_prev = pad_2_axlr

.segment "ZEROPAGE"
  note_playing:                 .res 4  ; N bit is for off
  note_type:                    .res 4  ; iit-pbbb
  note_phase:                   .res 4

  p1_base_note:                 .res 1  ; N is octave toggle
  p2_base_note:                 .res 1  ;

  prev_p1_last_keys:            .res 1
  prev_p2_last_keys:            .res 1
  prev_pl1_hi:                  .res 1
  prev_pl2_hi:                  .res 1

  p1_octave_toggle:             .res 1
  p2_octave_toggle:             .res 1

  pad_1_lockout:                .res 1
  pad_2_lockout:                .res 1

; TODO: obsolete
  sq1_note:                     .res 1  ; N bit is for silence
  sq2_note:                     .res 1  ; ^
  tri_note:                     .res 1  ; ^
  sq_toggle:                    .res 1
  sq1_button:                   .res 1  ; N bit is for empty
  sq2_button:                   .res 1  ; ^
  tri_button:                   .res 1  ; ^

.segment "CODE"

.proc controller_music
  jsr ppu_vsync_forced_blank

  jsr ppustrings_init

  ldy #<$3fe0
  lda #>$3fe0
  jsr ppu_set_addr
  ;,; ldy #$e0
  set_pal_loop:
    lda controller_music_pal-$e0, y
    sta PPU_DATA
    iny
  bne set_pal_loop

  ;,; ldy #$00
  @__loop1:
    lda controller_music_sprites, y
    sta ppu_oam, y
    iny
    cpy #(4*12)
  bcc @__loop1

  ldy #<$0000
  lda #>$0000
  jsr ppu_set_addr

  ldy #<controller_music_chr
  lda #>controller_music_chr
  jsr bagel_bulk_upload

  ldy #<$2000
  lda #>$2000
  jsr ppu_set_addr

  ldy #<controller_music_nam
  lda #>controller_music_nam
  jsr ntd_decode
  
  lda #$00
  sta ppu_var_scroll_x
  lda #240-3
  sta ppu_var_scroll_y
  lda #VBLANK_NMI|BG_0000|OBJ_0000|NT_2800
  sta ppu_var_ctrl
  lda #BG_ON|OBJ_ON
  sta ppu_var_mask

  lda #$ff
  sta prev_pl1_hi
  sta prev_pl2_hi

  ldy #<$23c0
  lda #>$23c0
  jsr ppustrings_new_string
  ldy #64
  lda #$00
  @__loop2:
    sta ppu_buffer, x
    inx
    dey
  bne @__loop2
  jsr ppustrings_finish_string

  jsr stack_restoreState

  lda #(2*12)
  sta p1_base_note
  lda #(3*12)
  sta p2_base_note
  lda #$ff
  sta pad_1_keys
  sta pad_2_keys

main_loop:
  jsr copy_keys_to_prev
  jsr controller_app_ppu_update
  inc ppustrings_ready
  jsr ppu_vsync
  jsr ppustrings_nmi_kernel
  jsr controller_app_sound_update
jmp main_loop
rts
.endproc

.proc copy_keys_to_prev
  lda pad_1_keys
  eor #$ff
  sta pad_1_inv_prev
  ora pad_1_lockout
  sta pad_1_lockout
  lda pad_2_keys
  eor #$ff
  sta pad_2_inv_prev
  ora pad_2_lockout
  sta pad_2_lockout
rts
.endproc

.proc controller_app_sound_update
  lda pad_1_keys
  and pad_1_inv_prev
  and #BUTTON_SELECT
  beq dont_toggle_p1_octave
    lda p1_octave_toggle
    eor #12
    sta p1_octave_toggle
  dont_toggle_p1_octave:

  lda pad_2_keys
  and pad_2_inv_prev
  and #BUTTON_SELECT
  beq dont_toggle_p2_octave
    lda p2_octave_toggle
    eor #12
    sta p2_octave_toggle
  dont_toggle_p2_octave:

  lda #$80
  sta sq1_note
  lda pad_1_keys
  and pad_1_lockout
  and #$ff^BUTTON_SELECT
  beq no_buttons_pressed
    ldy #$100-1
    read_button_loop:
      iny
      asl
    bcc read_button_loop
    clc
    lda button_to_note_number_table, y
    adc #(2*12)
    clc
    adc p1_octave_toggle
    sta sq1_note
  no_buttons_pressed:

  lda #$80
  sta tri_note
  lda pad_2_keys
  and pad_2_lockout
  and #$ff^BUTTON_SELECT
  beq no_buttons_pressed2
    ldy #$100-1
    read_button_loop2:
      iny
      asl
    bcc read_button_loop2
    clc
    lda button_to_note_number_table, y
    adc #(3*12)
    clc
    adc p2_octave_toggle
    sta tri_note
  no_buttons_pressed2:

  lda #$0f
  sta APU_PL1_SWEEP

  ldy #%10110000
  lda sq1_note
  bmi sound_is_off
    ldy #%10111111
  sound_is_off:
  sty APU_PL1_VOL

  ldy #%00000000
  lda tri_note
  bmi sound_is_off2
    ldy #%11111111
  sound_is_off2:
  sty APU_TRI_LINEAR

  ldy sq1_note
  lda tv_type
  jsr caculate_pitch
  sty APU_PL1_LO
  cmp prev_pl1_hi
  beq skip_hi
    sta APU_PL1_HI
    sta prev_pl1_hi
  skip_hi:

  ldy tri_note
  lda tv_type
  jsr caculate_pitch
  sty APU_TRI_LO
  sta APU_TRI_HI

;  lda #$0f
;  sta APU_PL1_SWEEP
;  sta APU_PL2_SWEEP
;  ldy #%00111111
;  sty APU_PL1_VOL
;  sty APU_PL2_VOL

;  ldy sq1_note
;  iny
;  cpy #8
;  bcc no_warp
;    ldy #0
;  no_warp:
;  sty sq1_note
;  lda temp_arp_table, y
;  tay
;  lda tv_type
;  jsr caculate_pitch
;  sty APU_PL1_LO
;  cmp prev_pl1_hi
;  beq skip_hi4
;    sta APU_PL1_HI
;    sta prev_pl1_hi
;  skip_hi4:
;
;  ldy sq2_note
;  iny
;  cpy #8
;  bcc no_warp2
;    ldy #0
;  no_warp2:
;  sty sq2_note
;  lda temp_arp_table2, y
;  tay
;  lda tv_type
;  jsr caculate_pitch
;  sty APU_PL2_LO
;  cmp prev_pl2_hi
;  beq skip_hi3
;    sta APU_PL2_HI
;    sta prev_pl2_hi
;  skip_hi3:
rts
.endproc

temp_arp_table:
  .byte  3+36, 3+36, 3+36, 3+36,10+36,10+36,10+36,10+36
temp_arp_table2:
  .byte  0+48, 0+48, 7+36, 7+36, 7+36, 7+36, 0+48, 0+48

; TODO: make Select change octave, not G#
button_to_note_number_table:
  .byte 10, 5, 12, 0, 8, 7, 2, 3
; A  B  C  D  E  F  G  A-2
; St ←  →  B  ↓  ↑  A  Sl

.import stack_dup, stack_drop
.import stack_increment, stack_decrement, stack_shiftRight
.import stack_add, stack_multiply
;.import rng_stepBit, rng_stepByte

;;
;   In: ( (0:note,1:tv_type) )
;  Out: ( apu_period )
.proc caculate_pitch
  jsr stack_dup
  lda #0
  ldy #8
  asl 0,x
  loop1:
    rol
    cmp #12
    bcc loop2
      sbc #12
    loop2:
    rol 0,x
    dey
  bne loop1
  tay

  lda 0,x
  pha
    dex
    dex
    lda period_multipliers_table_lo, y
    sta 0,x
    lda period_multipliers_table_hi, y
    sta 1,x

    ldy 3,x
    lda base_pitch_lo, y
    sta 2,x
    lda base_pitch_hi, y
    sta 3,x

    jsr stack_drop
    jsr stack_multiply
;    jsr stack_dup

;    jsr rng_stepByte
;    jsr rng_stepByte
;    ;; jsr stack_push_ya
;    jsr stack_add
    jsr stack_drop
;    bcc @__skip3347
;      jsr stack_increment
;    @__skip3347:
    jsr stack_dup
  pla
  clc
  adc #4
  sta temp+2
  @__loop2113:
;    jsr rng_stepBit
    jsr stack_drop
;    bcc @__skip3426
;      jsr stack_increment
;    @__skip3426:
    jsr stack_shiftRight
    jsr stack_dup
    dec temp+2
  bpl @__loop2113
  jsr stack_drop
  jsr stack_decrement
rts
; ntsc_cpu = ((236250000 / 11) / 12)
; pal_cpu = ((266017125 / 10) / 16)
; dendy_cpu = ((266017125 / 10) / 15)
; A1 = (cpu / (16 * (440 * (2 ** (-36 / 12)))))*32
; A1-100cents = (cpu / (16 * (440 * (2 ** (-36.1 / 12)))))*32
base_pitch_lo:
  .byte <65460, <60809, <64863
base_pitch_hi:
  .byte >65460, >60809, >64863

; for i in range(12):
;     round((2 ** ((-i-0.1) / 12))*65536)
period_multipliers_table_lo:
  .byte <65159, <61501, <58050, <54792, <51716, <48814
  .byte <46074, <43488, <41047, <38743, <36569, <34517
period_multipliers_table_hi:
  .byte >65159, >61501, >58050, >54792, >51716, >48814
  .byte >46074, >43488, >41047, >38743, >36569, >34517

;; [ ((2 ** (math.cos((math.pi*2)*(i+0.5)/7)))) for i in range(6) ]
;; [ ((2 ** (math.cos((math.pi*2)*(i+0.5)/6)))) for i in range(5) ]
;vibrato_multipliers_table_lo:
;  .byte <65159, <61501, <58050, <54792, <51716
;  .byte <46074, <43488, <41047, <38743, <36569
;vibrato_multipliers_table_hi:
;  .byte >65159, >61501, >58050, >54792, >51716
;  .byte >46074, >43488, >41047, >38743, >36569
.endproc

.proc controller_app_ppu_update
gfx_ops = ppu_update_gfx_ops
  txa
  pha
    lda pad_1_keys
    and pad_1_lockout
    sta temp+0
    lda pad_2_keys
    and pad_2_lockout
    sta temp+1
    ldy #0
    change_attributes_loop:
      asl temp+1
      rol temp+0

      ldx gfx_ops+0, y
      bmi do_full_attr
      do_masked_attr:
        lda ppu_buffer+3, x
        and gfx_ops+1, y
        bcc masked_attr_not_on
          ora gfx_ops+2, y
        masked_attr_not_on:
        sta ppu_buffer+3, x
      jmp done
      do_full_attr:
        ldx gfx_ops+1, y
        bmi do_oam
          lda #$55
          bcc full_attr_not_on
            lda #$ff
          full_attr_not_on:
          sta ppu_buffer+3, x
          ldx gfx_ops+2, y
          sta ppu_buffer+3, x
        jmp done2
        do_oam:
          ldx gfx_ops+2, y
          lda #%00000000
          bcc oam_not_on
            lda #%00000001
          oam_not_on:
          sta ppu_oam+2, x
          sta ppu_oam+2+4, x
          ora #%01000000
          sta ppu_oam+2+8, x
        done2:
      done:
      iny
      iny
      iny
      cpy #16*3
    bcc change_attributes_loop
  pla
  tax
rts
.endproc

;stack_fetch_block:    ; ( a-addr u -- ... )
;  lda 0,x
;  sta temp+0
;  lda 1,x
;  sta temp+1
;  inx
;  inx
;  @copy_loop:
;    dey
;  bmi @end_loop
;    lda (temp), y
;    dex
;    sta 0,x
;  jmp @copy_loop
;  @end_loop:
;jmp stack_drop

.segment "RODATA"

controller_music_sprites:
  .byte 82, 5, %00000000, 96
  .byte 82, 1, %00000000, 102
  .byte 82, 5, %01000000, 107

  .byte 82, 5, %00000000, 125
  .byte 82, 1, %00000000, 130
  .byte 82, 5, %01000000, 136

  .byte 194, 5, %00000000, 96
  .byte 194, 1, %00000000, 102
  .byte 194, 5, %01000000, 107

  .byte 194, 5, %00000000, 125
  .byte 194, 1, %00000000, 130
  .byte 194, 5, %01000000, 136

ppu_update_gfx_ops:
  .byte $ff, $0e, $16
  .byte $ff, $0d, $15
  .byte $ff, $ff, (4*0)
  .byte $ff, $ff, (4*3)
  .byte $09, $3f, $80
  .byte $11, $3f, $80
  .byte $11, $fc, $02
  .byte $12, $fc, $02
  .byte $ff, $2e, $36
  .byte $ff, $2d, $35
  .byte $ff, $ff, (4*6)
  .byte $ff, $ff, (4*9)
  .byte $29, $f3, $08
  .byte $31, $f3, $08
  .byte $29, $cf, $20
  .byte $2a, $cf, $20

; TODO: compressed pal idea.
;controller_music_pal:
;  .byte %00111111  ; each bit is for what pallets to change
;  .byte $0f,$10,$20, $16,$26,$20, $2a,$3a,$20, $28,$38,$20
;  .byte $0f,$00,$10, $12,$22,$32

controller_music_pal:
  .byte $00,$0f,$10,$20, $00,$16,$26,$20, $00,$2a,$3a,$20, $00,$28,$38,$20
  .byte $00,$0f,$00,$10, $00,$12,$22,$32, $00,$00,$00,$00, $00,$00,$00,$00

controller_music_nam:
  .incbin "src/screens/controller-test.nam.ntd"

controller_music_chr:
  .incbin "obj/controller-music.chr.bagel"

