; NES Init Code
; Copyright 2015 Johnathan Roatch
;
; Copying and distribution of this file, with or without
; modification, are permitted in any medium without royalty provided
; the copyright notice and this notice are preserved in all source
; code copies.  This file is offered as-is, without any warranty.
;

.include "nes.inc"

.export init_controller_music

.importzp temp
.importzp nmi_count, tv_type
.importzp stack_tos

.import rng_init, ppu_clear_oam_nocheck, ppu_set_addr, ppu_forced_blank
.import controller_music, button_logger_activity

.segment "CODE"

.proc init_controller_music
  sei                   ; Disable IRQ
  ldy #%00000000
  sty PPU_CTRL          ; Disable NMI
    ; Stop IRQ and NMI interrupts ASAP.

; Putting other sources of interrupts into a known state.
  sty PPU_MASK          ; Disable rendering
  sty APU_DMC_FREQ      ; Disable DMC IRQ
  bit APU_STATUS        ; Acknowledge DMC IRQ
  lda #%01000000
  sta APU_FRAMECNT      ; Disable APU Frame IRQ
  lda #%00001111
  sta APU_SND_CHN       ; Disable DMC playback, initialize other channels
  cld                   ; Turn off decimal mode for compatibility.

; Setting up for the cycle timed loops.
  ;,; ldy #0
  ldx #$100-1
  lda #24
; Acknowledge stray vblank NMI across reset
  bit PPU_STATUS
; It takes one full frame for the PPU to become stable.
; Best practice is to spin twice until the vblank flag of
; PPU_STATUS is set.
  vwait1:
    bit PPU_STATUS
  bpl vwait1

; We have cycles to burn until the second frame's vblank.
; Use this time to detect what tv system we're on.
; PPU_STATUS is checked after these amount of cycles
; but before each of their vblank ends.
;   NTSC NES: 29780 cycles per frame (2273 for vblank)
;   PAL NES:  33247 cycles per frame (7459 for vblank)
;   Dendy:    35464 cycles per frame (2273 for vblank)
; I end up hitting around cycles 30870, 33440, 36020.
  vwait2:
    @multiple_loop:
      @1279_cycles:
        dey
      bne @1279_cycles
      sec
      sbc #1
    bne @multiple_loop
    inx
    lda #2
    bit PPU_STATUS
  bpl vwait2
    ; After the second vblank, we know the PPU has fully stabilized,
    ; and that the TV type is in X.

  stx tv_type
  ldx #$ff
  txs
    ; Finally! We can now touch the RAM and stack pointer.

  cli

  jsr ppu_forced_blank

  ;,; ldy #$00
  jsr ppu_clear_oam_nocheck

;  jsr rng_init
    ; I know some bytes in zeropage already got set to a known
    ; state thereby reducing entropy, but hopfully leftover
    ; OAM will have enough anyway.

  lda #0
  ldx #$f0
  clear_zp_loop:
    dex
    sta 0, x
  bne clear_zp_loop

  ldx #<$f0-2
  stx stack_tos
    ; initalize data stack pointer.

jmp controller_music
.endproc

