; 6502 data stack operations
; Copyright 2015 Johnathan Roatch
;
; Copying and distribution of this file, with or without
; modification, are permitted in any medium without royalty provided
; the copyright notice and this notice are preserved in all source
; code copies.  This file is offered as-is, without any warranty.
;
; TODO: explain this section detail
; Standard calling conventions
; Y = TOS high byte.
; A = TOS low byte.
; X = Data Stack pointer
;
; data stack ordering conventions:
;   for 32-bit numbers, the least significant word is pushed last.
;

.importzp temp
.exportzp stack_tos

.export stack_saveState, stack_restoreState
.export stack_pushZero, stack_pushNegOne
.export stack_fetch, stack_store
.export stack_drop, stack_dup, stack_swap, stack_over, stack_nip, stack_tuck
.export stack_add, stack_subtract, stack_negate, stack_increment, stack_decrement
.export stack_2dup, stack_2drop, stack_increment2ndItem, stack_2plus
.export stack_bitwiseAnd, stack_bitwiseOr, stack_bitwiseXor
.export stack_bitwiseInvert
;.export stack_bitwiseMajority
.export stack_shiftLeft, stack_shiftRight, stack_arithmeticShiftRight
.export stack_multiply, stack_divide, stack_udslashmod
.export stack_jumpToSubroutine

.segment "ZEROPAGE": zeropage
  stack_tos:        .res 1

.segment "CODE"

.scope stack

N = temp

save_stack_ptr:
  jsr dup
  stx stack_tos
rts

restore_stack_ptr:
  ldx stack_tos
jmp drop

dup:                ; ( n1 -- n1 n1 )
  dex
  dex
  sta 1,x
  sty 0,x
rts

_2dup:               ; ( d1 -- d1 d1 )
  jsr over
over:               ; ( n1 n2 -- n1 n2 n1 )
  jsr dup
  ldy 2,x
  lda 3,x
rts

_2drop:             ; ( d1 -- )
  inx
  inx
drop:               ; ( n1 -- )
  ldy 0,x
  lda 1,x
nip:                ; ( n1 n2 -- n2 )
  inx
  inx
rts

tuck:               ; ( n1 n2 -- n2 n1 n2 )
  jsr swap
jmp over

max:                ; ( n1 n2 -- max(n1,n2) )
  jsr _2dup
  jsr gt            ; if n1>n2
  jmp _drop_choice
min:                ; ( n1 n2 -- min(n1,n2) )
  jsr _2dup
  jsr lt            ; if n1<n2
_drop_choice:
  ;,; cpy #$00      ; if flag is true, then drop flag and n2
  bne _2drop        ; else drop flag and nip n1
drop_swap_drop:     ; ( n1 n2 n3 -- n2 )
  jsr drop
  inx
  inx
rts

rot:                ; ( n1 n2 n3 -- n2 n3 n1 )
;Forth definition;  : >R SWAP R> SWAP ;
  pha
    tya
    pha
      jsr drop
      jsr swap
      jsr dup
    pla
    tay
  pla
swap:               ; ( n1 n2 -- n2 n1 )
  dex
  pha
    tya
    jsr @_swap_byte
    tay
  pla
  inx
@_swap_byte:
  eor 1,x
  sta N
  eor 1,x
  sta 1,x
  eor N
rts

add:
plus:               ; ( n1 n2 -- n1+n2 )
  clc
_add_with_carry:
  pha
    tya
    adc 0,x
    tay
  pla
  adc 1,x
  inx
  inx
rts

subtract:
minus:              ; ( n1 n2 -- n1-n2 )
  sec  ; add 1 to turn into two's complement
_subtract_with_carry:
  pha
    tya
    eor #$ff
    adc 0,x
    tay
  pla
  eor #$ff
  adc 1,x
  inx
  inx
rts

abs:                ; ( n -- |n| )
  cmp #$00
  bpl _abs_done
negate:             ; ( n -- -n )
  jsr zero
reverse_subtract:   ; ( n1 n2 -- -n1+n2)
  pha
    tya
    sec
    sbc 0,x
    tay
  pla
  sbc 1,x
  inx
  inx
_abs_done:
rts

gt:                 ; ( n1 n2 -- f )  f=true if n1>n2.
  jsr reverse_subtract
jmp _convert_subtraction_to_flag
lt:                 ; ( n1 n2 -- f )  f=true if n1<n2.
  jsr subtract
_convert_subtraction_to_flag:
  ldy #$ff
  cmp #$00
  bvc @_not_v
    eor #$80
  @_not_v:
  bmi @_n1_is_lt
    iny
  @_n1_is_lt:
  tya
rts

leq:                ; ( n1 n2 -- f )  f=true if n1<=n2.
  jsr gt
jmp not

geq:                ; ( n1 n2 -- f )  f=true if n1>=n2.
  jsr lt
jmp not

shift_left:
  pha
    tya
    asl
    tay
  pla
  rol
rts

shift_right:
  lsr
  bpl _ror_low_byte  ;,; jmp @_ror_low_byte
arithmetic_shift_right:
  cmp #$80
  ror
_ror_low_byte:
  pha
    tya
    ror
    tay
  pla
rts

false:
zero:               ; ( -- 0 )
  jsr dup
  ldy #0
  tya
rts

one:                ; ( -- 1 )
  jsr dup
  ldy #1
  lda #0
rts

true:
neg1:               ; ( -- -1 )
  jsr dup
  ldy #$ff
  tya
rts

increment:          ; ( n -- n+1 )
  iny
  bne @__skip
    clc
    adc #$01
  @__skip:
rts

increment_2nd_item: ; ( n1 n2 -- n1+1 n2 )
  inc 0,x
  bne @__skip
    inc 1,x
  @__skip:
rts

decrement:          ; ( n -- n-1 )
  cpy #$00
  bne @__skip
    sec
    sbc #$01
  @__skip:
  dey
rts

_and:               ; ( n1 n2 -- n1&n2 )
  pha
    tya
    and 0,x
    tay
  pla
  and 1,x
  inx
  inx
rts

or:                 ; ( n1 n2 -- n1|n2 )
  pha
    tya
    ora 0,x
    tay
  pla
  ora 1,x
  inx
  inx
rts

not:                ; ( n -- ~n )
  jsr neg1
xor:                ; ( n1 n2 -- n1^n2 )
  pha
    tya
    eor 0,x
    tay
  pla
  eor 1,x
  inx
  inx
rts

fetch:              ; ( addr -- n )
  jsr dup
  lda (0,x)
  tay
  jsr increment_2nd_item
  lda (0,x)
  inx
  inx
rts

c_fetch:            ; ( addr -- b )
  jsr dup
  lda (0,x)
  tay
  lda #$00
  inx
  inx
rts

store:              ; ( n addr -- )
  jsr swap
  pha
    tya
    sta (0,x)
  pla
  jsr increment_2nd_item
  sta (0,x)
jmp _2drop

c_store:            ; ( b addr -- )
  jsr swap
  tya
  sta (0,x)
jmp _2drop

u_star:             ; ( u1 u2 -- u1*u2 )
  jsr um_star
  inx
  inx
rts

; http://forum.6502.org/viewtopic.php?p=4389#p4389
um_star:
multiply_unsigned:  ; ( u1 u2 -- ud )
  jsr dup
  lda #0
  sta N
  ldy #16
  lsr 1,x
  ror 0,x
  @__loop:
    bcc @__skip
      clc
      pha
        lda N
        adc 2,x
        sta N
      pla
      adc 3,x
    @__skip:
    ror
    ror N
    ror 1,x
    ror 0,x
    dey
  bne @__loop
  sta 3,x
  lda N
  sta 2,x
jmp drop

; http://forum.6502.org/viewtopic.php?p=39595#p39595
;------------------------------------------------------
; UM/MOD for any 6502 by Michael T. Barry.
; Free to copy and/or modify, but without warranty.
;
; ( ud u1 -- u2 u3 )
; ud is the 32-bit dividend.
; u1 is the 16-bit divisor.
; u2 is the 16-bit remainder.
; u3 is the 16-bit quotient.
; All values are unsigned.
;
; Invalid results for quotient overflow or /0
umslashmod:         ; ( ud u1 -- u2 u3 )
  jsr dup
  ldy #16
  @__loop:
    asl 2,x         ; dividend:L is gradually replaced
    rol 3,x         ;   with the quotient
    rol 4,x         ; dividend:H is gradually replaced
    rol 5,x         ;   with the remainder
    bcs @do_sub
      lda 4,x       ; compare remainder to divisor
      cmp 0,x
      lda 5,x
      sbc 1,x
    bcc @__skip     ; if (remainder >= divisor) then
    @do_sub:        ;   update partial remainder
      ;,; sec
      lda 4,x
      sbc 0,x
      sta 4,x
      lda 5,x
      sbc 1,x
      sta 5,x
      inc 2,x       ; set low bit in quotient
    @__skip:
    dey
  bne  @__loop
jmp _2drop          ; drop the temp counter and divisor

dplus:              ; ( d1 d2 -- d1+d2 )
  jsr dup
  clc
  ldy #4
  @__loop:
    lda 4,x
    adc 0,x
    sta 4,x
    inx
    dey
  bne @__loop
jmp drop

dminus:             ; ( d1 d2 -- d1-d2 )
  jsr dup
  sec
  ldy #4
  @__loop:
    lda 4,x
    sbc 0,x
    sta 4,x
    inx
    dey
  bne @__loop
jmp drop

udslashmod:         ; ( ud1 u1 -- u2 ud2 )
  jsr zero          ; ( hi lo div 0 )
  jsr dup
  ldy 6,x
  lda 7,x           ; ( hi lo div 0 hi )
  jsr dup
  ldy 4,x
  lda 5,x           ; ( hi lo div 0 hi div )
  jsr umslashmod    ; ( hi lo div rem_hi quo_hi )
  pha
    tya
    pha
      jsr drop      ; ( hi lo div rem_hi , R: quo_hi )
      sty 4,x
      sta 5,x
      jsr drop      ; ( rem_hi lo div , R: quo_hi )
      jsr umslashmod ; ( rem_lo quo_lo , R: quo_hi )
      jsr dup
    pla
    tya
  pla               ; ( rem_lo quo_lo quo_hi )
jmp swap            ; ( rem_lo quo_hi quo_lo )

;; this used to be useful for correcting multiple controller reads.
;bitwise_majority:   ; ( a b c -- <a,b,c> )
;  jsr dup
;  jsr @bitwise_majority_subroutine
;  jsr @bitwise_majority_subroutine
;  inx
;  inx
;jmp drop
;@bitwise_majority_subroutine:
;  ; c = (a&b)|(b&c)|(c&a)
;  lda 4,x
;  tay
;    and 0,x
;    sta 4,x     ; (a&c)
;  tya
;  and 2,x
;  ora 4,x
;  sta 4,x       ; | (c&b)
;  lda 0,x
;  and 2,x
;  ora 4,x
;  sta 4,x       ; | (a&b)
;  inx
;rts

split_bytes:        ; ( n -- b_lo b_hi )
  dex
  dex
  sta 0,x           ; Put top high byte (A) into next low byte
                    ; top low byte (Y) remains the same.
  lda #$00          ; Zero the high byte of both cells.
  sta 1,x
rts

combine_bytes:      ; ( b1 b2 -- b1*256+b2 )
  lda 0,x
  inx
  inx
rts

swap_bytes:         ; ( <l,h> -- <h,l> )
;Forth definition;  : split_bytes swap combine_bytes ;
  sta N
  tya
  ldy N
rts

execute:            ; ( addr -- ) / addr -> PC
  jsr decrement
execute_subroutine: ; ( s-addr -- ) / s-addr+1 -> PC
  pha
  tya
  pha
jmp drop
.endscope

stack_saveState = stack::save_stack_ptr
stack_restoreState = stack::restore_stack_ptr
stack_pushZero = stack::zero
stack_pushNegOne = stack::neg1
stack_fetch = stack::fetch
stack_store = stack::store
stack_drop = stack::drop
stack_dup = stack::dup
stack_nip = stack::nip
stack_tuck = stack::tuck
stack_over = stack::over
stack_swap = stack::swap
stack_add = stack::add
stack_subtract = stack::subtract
stack_negate = stack::negate
stack_increment = stack::increment
stack_decrement = stack::decrement
stack_2dup = stack::_2dup
stack_2drop = stack::_2drop
stack_2plus = stack::dplus
stack_increment2ndItem = stack::increment_2nd_item
stack_multiply = stack::multiply_unsigned
stack_divide = stack::umslashmod
stack_udslashmod = stack::udslashmod
;stack_bitwiseMajority = stack::bitwise_majority
stack_bitwiseAnd = stack::_and
stack_bitwiseOr = stack::or
stack_bitwiseXor = stack::xor
stack_bitwiseInvert = stack::not
stack_shiftLeft = stack::shift_left
stack_shiftRight = stack::shift_right
stack_arithmeticShiftRight = stack::arithmetic_shift_right
stack_jumpToSubroutine = stack::execute_subroutine
